# -*- coding: utf-8 -*-
from py2cytoscape.data.base_view import BaseView

import requests
import json

from . import HEADERS


class NodeView(BaseView):

    # Utility Methods to access node position
    def get_x(self):
        return self.get_value('NODE_X_LOCATION')

    def get_y(self):
        return self.get_value('NODE_Y_LOCATION')

    def set_x(self, x):
        self.set_value('NODE_X_LOCATION', x)







import uuid
import json
import os

# ######## Default Values ###########

# Define default widget size
DEF_HEIGHT = 700
DEF_WIDTH = 1098 # Same as cell width of Jupyter

DEF_BACKGROUND_COLOR = '#FFFFFF'

HTML_TEMPLATE_FILE = 'template.html'
STYLE_FILE = 'default_style.json'

# Default network
DEF_NODES = [
    {'data': {'id': 'Network Data'}},
    {'data': {'id': 'Empty'}}
]

DEF_EDGES = [
    {'data': {'id': 'is', 'source': 'Network Data', 'target': 'Empty'}}
]

DEF_LAYOUT = 'preset'
DEF_STYLE = 'default2'

PRESET_LAYOUTS = {
    'Preset': 'preset',
    'Circle': 'circle',
    'Concentric': 'concentric',
    'Breadthfirst': 'breadthfirst',
    'Spring': 'cose',
    'Grid': 'grid'
}


# Init styles
style_file = open(os.path.abspath(os.path.dirname(__file__)) + '/' + STYLE_FILE, 'r')
style_list = json.load(style_file)
STYLES = {}
for style in style_list:
    STYLES[style['title']] = style['style']


def render(network,
           style=DEF_STYLE,
           layout_algorithm=DEF_LAYOUT,
           background=DEF_BACKGROUND_COLOR,
           height=DEF_HEIGHT,
           width=DEF_WIDTH):
    """Render network data with embedded Cytoscape.js widget.

    :param network: dict (required)
        The network data should be in Cytoscape.js JSON format.
    :param style: str or dict
        If str, pick one of the preset style. [default: 'default']
        If dict, it should be Cytoscape.js style CSS object
    :param layout_algorithm: str
        Name of Cytoscape.js layout algorithm
    :param background: str
        Background in CSS format
    :param height: int
        Height of the widget.
    :param width: int
        Width of the widget.
    """

    from jinja2 import Template
    from IPython.core.display import display, HTML

    # Load style file if none available
    if isinstance(style, str):
        # Specified by name
        style = STYLES[style]

    if network is None:
        nodes = DEF_NODES
        edges = DEF_EDGES
    else:
        nodes = network['elements']['nodes']
        edges = network['elements']['edges']

    path = os.path.abspath(os.path.dirname(__file__)) + '/' + HTML_TEMPLATE_FILE
    template = Template(open(path).read())
    cyjs_widget = template.render(
        nodes=json.dumps(nodes),
        edges=json.dumps(edges),
        background=background,
        uuid="cy" + str(uuid.uuid4()),
        widget_width=str(width),
        widget_height=str(height),
        layout=layout_algorithm,
        style_json=json.dumps(style)
    )

    display(HTML(cyjs_widget))


# List of available layout algorithms
def get_layouts():
    return PRESET_LAYOUTS


def get_style_names():
    return list(STYLES.keys())


def get_style(name):
    if name in STYLES.keys():
        return STYLES[name]
    else:
        raise ValueError('Style does not exist: ' + name)


# -*- coding: utf-8 -*-
from py2cytoscape.data.base_view import BaseView
from . import BASE_URL

BASE_URL_NETWORK = BASE_URL + 'networks'


class EdgeView(BaseView):

    # Utility Methods to access node position
    def get_width(self):
        return self.__view['NODE_X_LOCATION']

    def get_color(self):
        return self.__view['NODE_Y_LOCATION']



# -*- coding: utf-8 -*-
import json

import pandas as pd
import numpy as np
import requests
from py2cytoscape.data.edge_view import EdgeView
from py2cytoscape.data.node_view import NodeView

from . import BASE_URL, HEADERS
from py2cytoscape.data.util_network import NetworkUtil

BASE_URL_NETWORK = BASE_URL + 'networks'


class CyNetworkView(object):

    def __init__(self, network=None, suid=None):
        if network is None:
            raise ValueError('Network is required.')

        # Validate required argument
        if pd.isnull(suid):
            raise ValueError("View SUID is missing.")
        else:
            self.__network = network
            self.__id = suid

        self.__url = BASE_URL_NETWORK + '/' \
                     + str(self.__network.get_id()) + '/views/' + str(self.__id)

    def get_id(self):
        """
        Get session-unique ID of this network view

        :return: SUID as integer
        """
        return self.__id

    def get_model_id(self):
        """
        Get network model SUID

        :return: model SUID as integer
        """
        return self.__network.get_id()

    def get_node_views(self):
        return self.__get_views('nodes')

    def get_edge_views(self):
        return self.__get_views('edges')

    def get_node_views_as_dict(self):
        return self.__get_views('nodes', format='dict')

    def get_edge_views_as_dict(self):
        return self.__get_views('edges', format='dict')

    def get_network_view_as_dict(self):
        return self.__get_views('network', format='dict')

    def get_node_views_as_dataframe(self):
        return self.__get_views('nodes', format='df')

    def get_edge_views_as_dataframe(self):
        return self.__get_views('edges', format='df')

    def __get_views(self, obj_type=None, format='view'):
        url = self.__url + '/' + obj_type
        views = requests.get(url).json()
        if format is 'dict':
            if obj_type is 'network':
                return self.__get_network_view_dict(views)
            else:
                return self.__get_view_dict(views)

        elif format is 'view':
            return self.__get_view_objects(views, obj_type)
        elif format is 'df':
            return _self.__get_view_df(views)
        else:
            raise ValueError('Format not supported: ' + format)

    def __get_view_objects(self, views, obj_type):
        view_list = []
        if obj_type is 'nodes':
            for view in views:
                view = NodeView(self, view['SUID'], obj_type)
                view_list.append(view)

        elif obj_type is 'edges':
            for view in views:
                view = EdgeView(self, view['SUID'], obj_type)
                view_list.append(view)

        else:
            raise ValueError('No such object type: ' + obj_type)

        return view_list

    def __get_view_dict(self, views):
        # reformat return value to simple dict
        view_dict = {}
        for view in views:
            key = view['SUID']
            values = view['view']
            # Flatten the JSON
            key_val_pair = {}
            for entry in values:
                vp = entry['visualProperty']
                value = entry['value']
                key_val_pair[vp] = value
            view_dict[key] = key_val_pair

        return view_dict

    def __get_view_df(self, views):
        # view_df depends on view_dict.
        view_dict = self.__get_view_dict(views)
        view_df = pd.DataFrame(view_dict)
        
        # Transpose the DataFrame to make the visual properties as columns
        view_df = view_df.T
        
        view_df.index.name = 'SUID'
        return view_df

    def __get_network_view_dict(self, values):
        # reformat return value to simple dict
        view_dict = {}
        # Flatten the JSON
        for entry in values:
            vp = entry['visualProperty']
            value = entry['value']
            view_dict[vp] = value

        return view_dict

    def update_node_views(self, visual_property=None, values=None, key_type='suid'):
        self.__update_views(visual_property, values, 'nodes', key_type)

    def batch_update_node_views(self, value_dataframe=None):
        self.__batch_update(value_dataframe, 'nodes')

    def batch_update_edge_views(self, value_dataframe=None):
        self.__batch_update(value_dataframe, 'edges')

    def update_edge_views(self, visual_property=None, values=None, key_type='suid'):
        self.__update_views(visual_property, values, 'edges', key_type)

    def update_network_view(self, visual_property=None, value=None):
        """
        Updates single value for Network-related VP.

        :param visual_property:
        :param value:
        :return:
        """

        new_value = [
            {
                "visualProperty": visual_property,
                "value": value
            }
        ]
        requests.put(self.__url + '/network', data=json.dumps(new_value),
                     headers=HEADERS)

    def __update_views(self, visual_property, values,
                       object_type=None, key_type='suid'):
        if key_type is 'name':
            name2suid = NetworkUtil.name2suid(self.__network)

        body = []
        for key in values.keys():
            if key_type is 'name':
                suid = name2suid[key]
                if suid is None:
                    continue
            else:
                suid = key

            new_value = self.__create_new_value(suid, visual_property,
                                                values[key])
            print(type(suid))

            body.append(new_value)


        requests.put(self.__url + '/' + object_type, json=body,
                     headers=HEADERS)

    def __create_new_value(self, suid, visual_property, value):
        if(isinstance( suid, np.int64 )):
            int_id = np.asscalar(suid)
        else:
            int_id = suid
        return {
            "SUID": int_id,
            "view": [
                {
                    "visualProperty": visual_property,
                    "value": value
                }
            ]
        }

    def __batch_update(self, df, object_type=None):
        body = []

        columns = df.columns
        for index, row in df.iterrows():
            entry = {
                'SUID': int(index),
                'view': self.__create_new_values_from_row(columns, row)
            }
            body.append(entry)

        requests.put(self.__url + '/' + object_type, data=json.dumps(body), headers=HEADERS)

    def __create_new_values_from_row(self, columns, row):
        views = []
        for column in columns:
            view = {
                "visualProperty": column,
                "value": row[column]
            }
            views.append(view)

        return views



# -*- coding: utf-8 -*-

import requests
import json

from . import HEADERS, SUID_LIST
from .style import Style


class StyleClient(object):

    def __init__(self, url):
        self.__url = url + 'styles'
        self.__url_apply = url + 'apply/styles/'

        self.vps = VisualProperties(url)

    def create(self, name=None, original_style=None):
        if name is None:
            raise ValueError('Name is required.')

        existing_styles = requests.get(self.__url).json()

        if name in existing_styles:
            return Style(name)

        style = {
            'title': name,
            'defaults': [],
            'mappings': []

        }
        new_style_name = requests.post(self.__url, data=json.dumps(style), headers=HEADERS).json()['title']
        return Style(name=new_style_name)

    def get_all(self):
        """
        Get list of all available styles.

        :return: List of style names
        """
        return requests.get(self.__url).json()

    def get(self, name, data_format='cy3'):
        if name is None:
            raise ValueError('Style name is required.')

        url = self.__url + '/' + name
        if data_format == 'cytoscapejs':
            url = url + '.json'
            return requests.get(url).json()[0]
        else:
            return requests.get(url).json()

    def apply(self, style, network=None):
        if network is None:
            raise ValueError('Target network is required')

        url = self.__url_apply + style.get_name() + '/' + str(network.get_id())
        requests.get(url)

    def delete(self, style):
        requests.delete(self.__url + '/' + style.get_name())

    def delete_all(self):
        requests.delete(self.__url)


class VisualProperties(object):

    def __init__(self, url):
        self.__url = url + 'styles/visualproperties'
        self.__convert_to_dict()

    def __convert_to_dict(self):
        vps = requests.get(self.__url).json()
        vp_dict = {}
        node_vps = []
        edge_vps = []
        network_vps = []

        for vp in vps:
            id = vp['visualProperty']
            name = vp['name']
            target_type = vp['targetDataType']
            if target_type == 'CyNode':
                node_vps.append(id)
            elif target_type == 'CyEdge':
                edge_vps.append(id)
            elif target_type == 'CyNetwork':
                network_vps.append(id)

            vp_dict[id] = name

        self.__vps = vp_dict
        self.__node_vp = tuple(node_vps)
        self.__edge_vp = tuple(edge_vps)
        self.__network_vp = tuple(network_vps)

    def get_all(self):
        return self.__vps

    def get_node_visual_props(self):
        return self.__node_vp

    def get_edge_visual_props(self):
        return self.__edge_vp

    def get_network_visual_props(self):
        return self.__network_vp







# -*- coding: utf-8 -*-
import copy

EMPTY_NETWORK = {
    'data': {
    },

    'elements': {
        'nodes': [],
        'edges': []
    }
}


def get_empty_network(name='Empty Network'):
    empty_network = copy.deepcopy(EMPTY_NETWORK)
    empty_network['data']['name'] = name
    return empty_network



import pandas as pd
from . import cytoscapejs as cyjs


def from_dataframe(df, source_col='source', target_col='target', interaction_col='interaction',
                   name='From DataFrame', edge_attr_cols=[]):
    """
    Utility to convert Pandas DataFrame object into Cytoscape.js JSON

    :param df:
    :param source_col:
    :param target_col:
    :param interaction_col:
    :param name:
    :param edge_attr_cols:
    :return:
    """
    network = cyjs.get_empty_network(name=name)
    nodes = set()

    for index, row in df.iterrows():
        s = row[source_col]
        t = row[target_col]
        if s not in nodes:
            nodes.add(s)
            source = get_node(s)
            network['elements']['nodes'].append(source)
        if t not in nodes:
            nodes.add(t)
            target = get_node(t)
            network['elements']['nodes'].append(target)

        network['elements']['edges'].append(get_edge(s, t, interaction=row[interaction_col]))
    return network


def to_dataframe(network, interaction='interaction', default_interaction='-'):
    edges = network['elements']['edges']

    network_array = []
    for edge in edges:
        edge_data = edge['data']
        source = edge_data['source']
        target = edge_data['target']
        if interaction in edge_data:
            itr = edge_data[interaction]
        else:
            itr = default_interaction
        row = (source, itr, target)
        network_array.append(row)

    return pd.DataFrame(network_array, columns=['source', 'interaction',
                                                'target'])


def get_node(id):
    node = {
        'data': {
            'id': str(id),
            'name': str(id)
        }
    }
    return node


def get_edge(source, target, interaction):
    if interaction is None:
        itr = '-'
    else:
        itr = interaction

    edge = {
        'data': {
            'source': source,
            'target': target,
            'interaction': itr
        }
    }

    return edge
